@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # Urwid SàT extensions
-# Copyright (C) 2009, 2010, 2011, 2012, 2013, 2014 Jérôme Poisson (goffi@goffi.org)
+# Copyright (C) 2009-2016 Jérôme Poisson (goffi@goffi.org)
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -18,16 +18,13 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 import urwid
-import sat_widgets
+from . import sat_widgets
 import os, os.path
 from xml.dom import minidom
-from logging import debug, info, error
+import logging as log
 from time import time
 from .keys import action_key_map as a_key
 
-import gettext
-gettext.install('urwid_satext', unicode=True)
-
 class PathEdit(sat_widgets.AdvancedEdit):
     """AdvancedEdit with manage file paths"""
 
@@ -91,13 +88,15 @@ class FilesViewer(urwid.WidgetWrap):
             return self._w.keypress(size, key)
 
     def showDirectory(self, path):
-        path = path.encode('utf-8')
         self.path = path
         del self.files_list[:]
         directories = []
         files = []
         try:
             for filename in os.listdir(path):
+                if not isinstance(filename, str):
+                    log.warning("file [{}] has a badly encode filename, ignoring it".format(filename.decode('utf-8', 'replace')))
+                    continue
                 fullpath = os.path.join(path,filename)
                 if os.path.isdir(fullpath):
                     directories.append(filename)
@@ -127,22 +126,30 @@ class FilesViewer(urwid.WidgetWrap):
             self.files_list.append(file_wid)
 
 
-
 class FileDialog(urwid.WidgetWrap):
 
-    def __init__(self, ok_cb, cancel_cb, title=_("Please select a file"), style=[]):
+    def __init__(self, ok_cb, cancel_cb, message=None, title=_("Please select a file"), style=[]):
         """Create file dialog
+
         @param title: title of the window/popup
+        @param message: message to display, or None to only show title and file dialog
+            message will be passed to a Text widget, so markup can be used
         @param style: list of string:
             - 'dir' if a dir path must be selected
         """
         self.ok_cb = ok_cb
         self._type = 'dir' if 'dir' in style else 'normal'
         self.__home_path = os.path.expanduser('~')
+        widgets = []
+        if message:
+            widgets.append(urwid.Text(message))
+            widgets.append(urwid.Divider('─'))
         self.path_wid = PathEdit(_('Path: '))
         self.path_wid.setCompletionMethod(self._directory_completion)
         urwid.connect_signal(self.path_wid, 'change', self.onPathChange)
-        header = urwid.Pile([self.path_wid, urwid.Divider(u'─')])
+        widgets.append(self.path_wid)
+        widgets.append(urwid.Divider('─'))
+        header = urwid.Pile(widgets)
         bookm_list = urwid.SimpleListWalker([])
         self.bookmarks = list(self.getBookmarks())
         self.bookmarks.sort()
@@ -166,7 +173,7 @@ class FileDialog(urwid.WidgetWrap):
         main_frame = sat_widgets.FocusFrame(center_row, header, buttons_wid)
         decorated = sat_widgets.LabelLine(main_frame, sat_widgets.SurroundedText(title))
         urwid.WidgetWrap.__init__(self, decorated)
-        self.path_wid.set_edit_text(os.getcwdu())
+        self.path_wid.set_edit_text(os.getcwd())
 
     def _validateDir(self, wid):
         """ call ok callback if current path is a dir """
@@ -175,6 +182,7 @@ class FileDialog(urwid.WidgetWrap):
             self.ok_cb(path)
 
     def _directory_completion(self, path, completion_data):
+        assert isinstance(path, str)
         path=os.path.abspath(path)
         if not os.path.isdir(path):
             head,dir_start = os.path.split(path)
@@ -183,6 +191,16 @@ class FileDialog(urwid.WidgetWrap):
             dir_start=''
         try:
             filenames = os.listdir(head)
+            to_remove = []
+
+            # we remove badly encoded files
+            for filename in filenames:
+                if not isinstance(filename, str):
+                    log.warning("file [{}] has a badly encode filename, ignoring it".format(filename.decode('utf-8', 'replace')))
+                    to_remove.append(filename)
+            for filename in to_remove:
+                filenames.remove(filename)
+
             filenames.sort()
             try:
                 start_idx=filenames.index(completion_data['last_dir'])+1
@@ -190,7 +208,7 @@ class FileDialog(urwid.WidgetWrap):
                     start_idx = 0
             except (KeyError,ValueError):
                 start_idx = 0
-            for idx in range(start_idx,len(filenames)) + range(0,start_idx):
+            for idx in list(range(start_idx,len(filenames))) + list(range(0,start_idx)):
                 full_path = os.path.join(head,filenames[idx])
                 if filenames[idx].lower().startswith(dir_start.lower()) and os.path.isdir(full_path):
                     completion_data['last_dir'] = filenames[idx]
@@ -207,9 +225,9 @@ class FileDialog(urwid.WidgetWrap):
             with open(gtk_bookm) as gtk_fd:
                 for bm in gtk_fd.readlines():
                     if bm.startswith("file:///"):
-                        bookmarks.add(bm[7:].replace('\n',''))
+                        bookmarks.add(bm[7:].replace('\n','').decode('utf-8', 'replace'))
         except IOError:
-            info(_('No GTK bookmarks file found'))
+            log.info(_('No GTK bookmarks file found'))
             pass
 
         try:
@@ -217,9 +235,9 @@ class FileDialog(urwid.WidgetWrap):
             for elem in dom.getElementsByTagName('bookmark'):
                 bm = elem.getAttribute("href")
                 if bm.startswith("file:///"):
-                    bookmarks.add(bm[7:])
+                    bookmarks.add(bm[7:].decode('utf-8', 'replace'))
         except IOError:
-            info(_('No KDE bookmarks file found'))
+            log.info(_('No KDE bookmarks file found'))
             pass
 
         return bookmarks
