@@ -19,8 +19,6 @@
 
 import urwid
 import logging as log
-import encodings
-utf8decode = lambda s: encodings.codecs.utf_8_decode(s)[0]
 
 import uuid
 
@@ -121,7 +119,7 @@ class Password(AdvancedEdit):
         """Same args than Edit.__init__ with an additional keyword arg 'hidden_char'
         @param hidden_char: char to show instead of what is actually entered: default '*'
         """
-        self.hidden_char=kwargs['hidden_char'] if kwargs.has_key('hidden_char') else '*'
+        self.hidden_char=kwargs['hidden_char'] if 'hidden_char' in kwargs else '*'
         self.__real_text=''
         super(Password, self).__init__(*args, **kwargs)
 
@@ -195,7 +193,7 @@ class SurroundedText(urwid.Widget):
     """Text centered on a repeated character (like a Divider, but with a text in the center)"""
     _sizing = frozenset(['flow'])
 
-    def __init__(self,text,car=utf8decode('─')):
+    def __init__(self,text,car='─'):
         self.text=text
         self.car=car
 
@@ -207,7 +205,7 @@ class SurroundedText(urwid.Widget):
 
     def display_widget(self, size, focus):
         (maxcol,) = size
-        middle = (maxcol-len(self.text))/2
+        middle = (maxcol-len(self.text))//2
         render_text = middle * self.car + self.text + (maxcol - len(self.text) - middle) * self.car
         return urwid.Text(render_text)
 
@@ -220,7 +218,7 @@ class AlwaysSelectableText(urwid.WidgetWrap):
         """
         @param text: same as urwid.Text's text parameter
         @param align: same as urwid.Text's align parameter
-        @select_attr: attrbute to use when selected
+        @param selected_text: text to display when selected
         @param selected: is the text selected ?
         """
         self.focus_attr = focus_attr
@@ -233,7 +231,7 @@ class AlwaysSelectableText(urwid.WidgetWrap):
         self.setState(selected)
 
     def getValue(self):
-        if isinstance(self.text,basestring):
+        if isinstance(self.text,str):
             return self.text
         list_attr = self.text if isinstance(self.text, list) else [self.text]
         txt = ""
@@ -255,7 +253,9 @@ class AlwaysSelectableText(urwid.WidgetWrap):
 
     def setSelectedText(self, text=None):
         """Text to display when selected
-        @text: text as in urwid.Text or None for default value"""
+
+        @text: text as in urwid.Text or None for default value
+        """
         if text == None:
             text = ('selected',self.getValue())
         self.selected_txt = text
@@ -369,7 +369,7 @@ class CustomButton(ClickableText):
         self.set_text([self.left_border, label, self.right_border])
 
 
-class ListOption(unicode):
+class ListOption(str):
     """Unicode which manage label and value
 
     This class similar to unicode, but which make the difference between value and label
@@ -383,7 +383,7 @@ class ListOption(unicode):
     def __new__(cls, option):
         if (isinstance(option, cls)):
             return option
-        elif isinstance(option, basestring):
+        elif isinstance(option, str):
             value = label = option
         elif (isinstance(option, tuple) and len(option) == 2):
             value, label = option
@@ -410,6 +410,9 @@ class ListOption(unicode):
         except AttributeError:
             return self._value != other
 
+    def __hash__(self):
+        return hash(self._value)
+
     @property
     def value(self):
         """ return option value """
@@ -515,7 +518,7 @@ class SimpleListWalkerWithCb(urwid.SimpleListWalker):
         self.__cbSingle(y, self._on_new)
         to_delete = parent.__getitem__(i)
         self.__cbSingle(to_delete, self._on_delete)
-        return parent.__setitem__(self, i, y)
+        return parent.__setitem__(i, y)
 
     def __setslice__(self, i, j, y):
         parent = super(SimpleListWalkerWithCb, self)
@@ -817,7 +820,7 @@ class NotificationBar(urwid.WidgetWrap):
                 self._modQueue()
                 return
 
-        raise ValueError(u"trying to remove an unknown pop_up_widget")
+        raise ValueError("trying to remove an unknown pop_up_widget")
 
     def addMessage(self, message):
         "Add a message to the notificatio bar"
@@ -956,7 +959,7 @@ class Menu(urwid.WidgetWrap):
         return self._w.base_widget.keypress(size, key)
 
     def checkShortcuts(self, key):
-        for shortcut in self.shortcuts.keys():
+        for shortcut in list(self.shortcuts.keys()):
             if key == shortcut:
                 category, item, callback = self.shortcuts[shortcut]
                 callback((category, item))
@@ -968,7 +971,7 @@ class Menu(urwid.WidgetWrap):
         @param category: category of the menu (e.g. File/Edit)
         @param item: menu item (e.g. new/close/about)
         @callback: method to call when item is selected"""
-        if not category in self.menu.keys():
+        if not category in list(self.menu.keys()):
             self.menu_keys.append(category)
             self.menu[category] = []
             button = CustomButton(('menubar',category), self.onCategoryClick,
@@ -979,7 +982,7 @@ class Menu(urwid.WidgetWrap):
             return
         self.menu[category].append((item, callback))
         if shortcut:
-            assert(shortcut not in self.shortcuts.keys())
+            assert(shortcut not in list(self.shortcuts.keys()))
             self.shortcuts[shortcut] = (category, item, callback)
 
     def onItemClick(self, widget):
@@ -1031,7 +1034,7 @@ class MenuRoller(urwid.WidgetWrap):
             self.columns.contents[1] = (urwid.Text(''), ('weight', 1, False))
         else:
             menu_item = self.menu_items[self.selected]
-            name_txt = u'\u21c9 ' + menu_item.name + u' \u21c7 '
+            name_txt = '\u21c9 ' + menu_item.name + ' \u21c7 '
             current_name = ClickableText(name_txt)
             name_len = len(name_txt)
             current_menu = menu_item.widget
@@ -1040,7 +1043,7 @@ class MenuRoller(urwid.WidgetWrap):
             self.columns.contents[1] = (current_menu, ('weight', 1, False))
 
     def keypress(self, size, key):
-        menu_ids = self.menu_items.keys()
+        menu_ids = list(self.menu_items.keys())
         try:
             idx = menu_ids.index(self.selected)
         except ValueError:
@@ -1074,7 +1077,7 @@ class MenuRoller(urwid.WidgetWrap):
         @param menu_id: id to use of this menu, or None to generate
         @return: menu_id
         """
-        names = {menu_item.name: id_ for id_, menu_item in self.menu_items.iteritems()}
+        names = {menu_item.name: id_ for id_, menu_item in self.menu_items.items()}
 
         if name not in names:
             id_ = menu_id or str(uuid.uuid4())
@@ -1109,13 +1112,13 @@ class MenuRoller(urwid.WidgetWrap):
         del self.menu_items[menu_id]
         if self.selected == menu_id:
             try:
-                self.selected = self.menu_items.iterkeys().next()
+                self.selected = next(iter(self.menu_items.keys()))
             except StopIteration:
                 self.selected = None
             self._showSelected()
 
     def checkShortcuts(self, key):
-        for menu_item in self.menu_items.values():
+        for menu_item in list(self.menu_items.values()):
             key = menu_item.widget.checkShortcuts(key)
         return key
 
@@ -1140,7 +1143,7 @@ class GenericDialog(urwid.WidgetWrap):
         if "OK" in style:
             self.buttons['ok'] = urwid.Button(_("Ok"), kwargs.get('ok_cb'), kwargs.get('ok_value'))
         if self.buttons:
-            buttons_flow = urwid.GridFlow(self.buttons.values(), max([len(button.get_label()) for button in self.buttons.itervalues()])+4, 1, 1, 'center')
+            buttons_flow = urwid.GridFlow(list(self.buttons.values()), max([len(button.get_label()) for button in self.buttons.values()])+4, 1, 1, 'center')
         body_content = urwid.SimpleListWalker(widgets_lst)
         frame_body = UnselectableListBox(body_content)
         frame = FocusFrame(frame_body, frame_header, buttons_flow if self.buttons else None, 'footer' if self.buttons else 'body')
@@ -1150,7 +1153,7 @@ class GenericDialog(urwid.WidgetWrap):
     def setCallback(self, name, callback, data=None):
         """Set the callback associated with a button press
 
-        @param name: one of "ok", "cancel", "yest", "no"
+        @param name: one of "ok", "cancel", "yes", "no"
         @aram callback(callable): method to call on requested action
         @param data: argument to send to the callback (first one will be the button widget)
         @raise KeyError if name is invalid
@@ -1166,10 +1169,14 @@ class InputDialog(GenericDialog):
         if style is None:
             style = ['OK/CANCEL']
         instr_wid = urwid.Text(instrucions+':')
-        edit_box = AdvancedEdit(edit_text=default_txt)
-        GenericDialog.__init__(self, [instr_wid,edit_box], title, style, ok_value=edit_box, **kwargs)
+        self.edit = AdvancedEdit(edit_text=default_txt)
+        GenericDialog.__init__(self, [instr_wid, self.edit], title, style, ok_value=self.edit, **kwargs)
         self._w.base_widget.focusposition = 'body'
 
+    @property
+    def text(self):
+        return self.edit.text
+
 
 class ConfirmDialog(GenericDialog):
     """Dialog with buttons for confirm or cancel an action"""
@@ -1324,9 +1331,9 @@ class ColumnsRoller(urwid.Widget):
             render.append((widget.render((width,),_focus),False,_focus,width))
             idx+=1
         if _prev:
-            render.insert(0,(urwid.Text([u"◀"]).render((1,),False),False,False,1))
+            render.insert(0,(urwid.Text(["◀"]).render((1,),False),False,False,1))
         if _next:
-            render.append((urwid.Text([u"▶"],align='right').render((1+cols_left,),False),False,False,1+cols_left))
+            render.append((urwid.Text(["▶"],align='right').render((1+cols_left,),False),False,False,1+cols_left))
         else:
             render.append((urwid.SolidCanvas(" "*cols_left, size[0], 1),False,False,cols_left))
 
@@ -1422,7 +1429,7 @@ class TabsContainer(urwid.WidgetWrap):
         self._current_tab = None
         self._buttons_cont = ColumnsRoller()
         self.tabs = []
-        self._frame = FocusFrame(urwid.Filler(urwid.Text('')),urwid.Pile([self._buttons_cont,urwid.Divider(u"─")]))
+        self._frame = FocusFrame(urwid.Filler(urwid.Text('')),urwid.Pile([self._buttons_cont,urwid.Divider("─")]))
         urwid.WidgetWrap.__init__(self, self._frame)
 
     def keypress(self, size, key):
@@ -1670,7 +1677,7 @@ class LabelLine(urwid.LineBox):
 
 
 class VerticalSeparator(urwid.WidgetDecoration, urwid.WidgetWrap):
-    def __init__(self, original_widget, left_char = u"│", right_char = ''):
+    def __init__(self, original_widget, left_char = "│", right_char = ''):
         """Draw a separator on left and/or right of original_widget."""
 
         widgets = [original_widget]
